program GetForecast;

(*

  Simple CGI app that calls ndfd library to get weather forecast data,
   which it returns as JSON (default), GeoJSON or JSON-P.

  For example, to retrieve today's (day 1) forecast for one location:
   http://localhost/cgi-bin/getforecast.cgi?lat1=37.81&lon1=-107.66

  Response will be formatted like this:
   [{"point":1,"day":1,"high":"19","low":"-5","display":"High: 19 Low: -5"}]

  Optional parameters:
   metric
   geojson
   callback
   day

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).

*)

{$mode objfpc}{$H+}

uses
  SysUtils,
  ezcgi,
  NdfdLib;

type
  TCgiForecast = class(TEZcgi)
    procedure DoGet; override;
  end;

procedure TCgiForecast.DoGet;
var
  Ndfd         : TNdfdLib;
  GetMetric    : Boolean;
  GetGeoJson   : Boolean;
  CallbackName : string;
  DayNum       : Integer;
  Response     : string;
  PtNum        : Integer;
  HighStr      : string;
  LowStr       : string;
begin

  Ndfd := TNdfdLib.Create;

  try
    GetMetric := Values['metric'] = 'true';

    GetGeoJson := Values['geojson'] = 'true';

    CallbackName := Values['callback'];

    DayNum := StrToIntDef(Values['day'], 1);

    Response := '';
    PtNum := 1;

    while Values['lat' + IntToStr(PtNum)] <> '' do
      begin
      HighStr := '';
      LowStr := '';
      if Ndfd.LoadForecast(Values['lat' + IntToStr(PtNum)],
                           Values['lon' + IntToStr(PtNum)],
                           GetMetric) then
        begin
        HighStr := Ndfd.GetMaxTemp(DayNum);
        LowStr := Ndfd.GetMinTemp(DayNum);
        end;

      if Response = '' then
        begin
        if GetGeoJson then
          Response := '{"type": "FeatureCollection", "features": [';
        end
      else
        Response := Response + ',';

      if GetGeoJson then
        Response := Response + '{"type": "Feature", "properties": ';

      Response := Response +
                  '{"point":' + IntToStr(PtNum) + ',' +
                   '"day":' + IntToStr(DayNum) + ',' +
                   '"high":"' + HighStr + '",' +
                   '"low":"' + LowStr + '",' +
                   '"display":"High: ' + HighStr + ' Low: ' + LowStr + '"}';

      if GetGeoJson then
        Response := Response + ', "geometry": {"type": "Point", "coordinates": [' +
                    Values['lon' + IntToStr(PtNum)] + ', ' +
                    Values['lat' + IntToStr(PtNum)] + '] } }';

      Inc(PtNum);
      end;

    if GetGeoJson then
      begin
      if Response <> '' then
        Response := Response + '] }';
      end
    else
      begin
      Response := '[' + Response + ']';  //Array
      if CallbackName <> '' then  {JSON-P callback function name passed?}
        Response := CallbackName + '(' + Response + ');';
      end;

    WriteContent('text/json');
    PutLine(Response);

  finally
    Ndfd.Free;
  end;

end;


var
  CgiForecast : TCgiForecast;

begin

  CgiForecast := TCgiForecast.Create;

  try
    try
      CgiForecast.Run;
    except on E:ECGIException do
      begin  //HTML error message has already been returned, so just ignore
      end;
    end;
  finally
    CgiForecast.Free;
  end;

end.
