program testgdal;

{$IFDEF FPC}
 {$MODE DELPHI}
{$ELSE}
 {$APPTYPE CONSOLE}
{$ENDIF}

uses
  SysUtils,
  ctypes,  {for Delphi: http://svn.freepascal.org/cgi-bin/viewvc.cgi/tags/release_2_6_4/rtl/inc/ctypes.pp?view=log}
  GDAL;
  
type
  SLarray    = array[0..32767] of Single;
var
  FName      : string;
  VectDS     : OGRDataSourceH;
  VectLayer  : OGRLayerH;
  RastDS     : GDALDatasetH;
  NumBands   : cint;
  BandNum    : Integer;
  RastRB     : GDALRasterBandH;
  RastMinMax : array [1..2] of Double;
  MinOkay    : cint;
  MaxOkay    : cint;
  RastCT     : GDALColorTableH;
  RastDType  : clong;
  nXSize     : cint;
  RastSL     : ^SLarray;  //pointer to scanline buffer
  RowNum     : Integer;
  ColNum     : Integer;
begin
  WriteLn(GDALVersionInfo('--version'));
  WriteLn(GDALVersionInfo('RELEASE_NAME'));
  WriteLn(GDALVersionInfo('RELEASE_DATE'));
  WriteLn(GDALVersionInfo('VERSION_NUM'));

   //defined in gdal_version.inc - this would be version of headers parsed
  WriteLn('Headers version: ', GDAL_VERSION_MAJOR, '.', GDAL_VERSION_MINOR, '.',
          GDAL_VERSION_REV);
//  WriteLn('Headers release date: ', GDAL_RELEASE_DATE);
  WriteLn;

  GDALAllRegister();
  OGRRegisterAll();

  if ParamCount = 0 then
    begin
    WriteLn('No file specified');
    Exit;
    end;

  FName := ParamStr(1);
//  FName := ExpandFileName(FName);
  if not FileExists(FName) then
    begin
    WriteLn('File does not exist');
    Exit;
    end;

  WriteLn('Opening ', FName, '...');
  VectDS := OGROpen(PAnsiChar(AnsiString(FName)), 0, nil);  {Try to open as a vector file}
  if Assigned(VectDS) then
    begin
    WriteLn('Layers: ', OGR_DS_GetLayerCount(VectDS));
    VectLayer := OGR_DS_GetLayer(VectDS, 0);
    WriteLn('Layer 1 name: ', OGR_L_GetName(VectLayer));
    WriteLn('Layer 1 features: ', OGR_L_GetFeatureCount(VectLayer, 1));
    OGRReleaseDataSource(VectDS);
    OGRCleanupAll;
    end
  else  {Unable to open as a vector file, so try opening as a raster file}
    begin
    RastDS := GDALOpen(PAnsiChar(AnsiString(FName)), GA_ReadOnly);
    if not Assigned(RastDS) then
      WriteLn('Unable to open file')
    else
      begin
      WriteLn('Columns: ', GDALGetRasterXSize(RastDS));
      WriteLn('Rows: ', GDALGetRasterYSize(RastDS));
      WriteLn;

      NumBands := GDALGetRasterCount(RastDS);
      WriteLn('Raster bands: ', NumBands);
      WriteLn('Raster color interps (3=red,4=green,5=blue,6=alpha,etc):');
      for BandNum := 1 to NumBands do
        WriteLn('  Band ', BandNum, ': ', 
                GDALGetRasterColorInterpretation(GDALGetRasterBand(RastDS, BandNum)));
      RastRB := GDALGetRasterBand(RastDS, 1);
      RastMinMax[1] := GDALGetRasterMinimum(RastRB, @MinOkay);
      RastMinMax[2] := GDALGetRasterMaximum(RastRB, @MaxOkay);
      if (MinOkay = 0) or (MaxOkay = 0) then
        GDALComputeRasterMinMax(RastRB, 1, @RastMinMax[1]);  //make sure 1 for bApproxOK is okay
      WriteLn('Raster band 1 minimum value: ', RastMinMax[1]:7:2);
      WriteLn('Raster band 1 maximum value: ', RastMinMax[2]:7:2);
      RastCT := GDALGetRasterColorTable(RastRB);
      if RastCT = nil then
        WriteLn('No color table');
      RastDType := GDALGetRasterDataType(RastRB);
      WriteLn('Raster band 1 data type: ', RastDType,
              ' and size: ', GDALGetDataTypeSize(RastDType) div 8, ' bytes');
      nXSize := GDALGetRasterBandXSize(RastRB);
      WriteLn('Raster X size: ', nXSize);
      GetMem(RastSL, SizeOf(Single)*nXSize);
       //Read and output a value from a cell in the middle of the grid:
      RowNum := GDALGetRasterYSize(RastDS) div 2; //numbering from 0
      ColNum := GDALGetRasterXSize(RastDS) div 2; //numbering from 0
      GDALRasterIO(RastRB, GF_Read, 0, RowNum, nXSize, 1, RastSL, nXSize, 1,
                   GDT_Float32, 0, 0);
      WriteLn('Raster band 1 scanline ', RowNum, ' pixel ', ColNum, ' value: ',
              RastSL^[ColNum]:7:2);
      FreeMem(RastSL);
      WriteLn;
      GDALClose(RastDS);
      end;
    end;

end.
