unit NdfdLib;

{
  Pascal wrapper class for ndfd library.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  SysUtils,
  ndfd_types;

{$DEFINE UNIV_OR_32BIT}
{$IFNDEF DARWIN}  {Assume OS X library is universal 32/64-bit}
  {$IFDEF CPUX64}
    {$UNDEF UNIV_OR_32BIT}
  {$ENDIF}
  {$IFDEF CPU64}
    {$UNDEF UNIV_OR_32BIT}
  {$ENDIF}
{$ENDIF}

const
{$IFDEF DARWIN}
 {$IFDEF NDFD_FRAMEWORK}
  {$linkframework NDFD}  {Link against NDFD.framework}
  NdfdLibNameBase = '';
 {$ELSE}
  {$linklib ndfd}  {Link against libndfd.dylib}
  NdfdLibNameBase = 'ndfd';
 {$ENDIF}
{$ELSE}
 {$IFDEF UNIV_OR_32BIT}
  NdfdLibNameBase = 'ndfd';
 {$ELSE}
  NdfdLibNameBase = 'ndfd64';
 {$ENDIF}
{$ENDIF}

 //By default, use cdecl on Windows too: if stdcall, MSVC won't link
 // to 32-bit Delphi/FPC DLL unless it exports decorated names too.
{$DEFINE USE_CDECL}
{$IFDEF USE_STDCALL}  //Override and use stdcall?
  {$UNDEF USE_CDECL}
{$ENDIF}

 {See ndfd.pas for description of each library function.}
procedure NdfdGetLibVersion(VerBuf    : NdfdPChar;
                            VerBufLen : NdfdUInt); {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
function NdfdInit : NdfdPObj; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
function NdfdUninit(NdfdObj : NdfdPObj) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
procedure NdfdGetLastError(NdfdObj   : NdfdPObj;
                           ErrBuf    : NdfdPChar;
                           ErrBufLen : NdfdUInt); {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
function NdfdLoadForecast(NdfdObj   : NdfdPObj;
                          Latitude  : NdfdPChar;
                          Longitude : NdfdPChar;
                          GetMetric : NdfdBool) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
function NdfdGetMaxTemp(NdfdObj    : NdfdPObj;
                        DayNum     : NdfdUInt;
                        TempBuf    : NdfdPChar;
                        TempBufLen : NdfdUInt) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;
function NdfdGetMinTemp(NdfdObj    : NdfdPObj;
                        DayNum     : NdfdUInt;
                        TempBuf    : NdfdPChar;
                        TempBufLen : NdfdUInt) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF}; external NdfdLibNameBase;

type
  TNdfdLib = class
  private
    NdfdObj : NdfdPObj;
    Buffer  : array [0..199] of NdfdChar;
  public
    constructor Create;
    destructor Destroy; override;
    function GetLibVersion : string;
    function GetLastError : string;
    function LoadForecast(const Latitude  : string;
                          const Longitude : string;
                                GetMetric : Boolean) : Boolean;
    function GetMaxTemp(DayNum : LongWord) : string;
    function GetMinTemp(DayNum : LongWord) : string;
  end;
  

implementation

constructor TNdfdLib.Create;
begin
  inherited Create;
  NdfdObj := NdfdInit;
end;

destructor TNdfdLib.Destroy;
begin
  NdfdUninit(NdfdObj);
  inherited Destroy;
end;

function TNdfdLib.GetLibVersion : string;
begin
  NdfdGetLibVersion(Buffer, SizeOf(Buffer));
  Result := CopyBufToStr(Buffer);
end;

function TNdfdLib.GetLastError : string;
begin
  NdfdGetLastError(NdfdObj, Buffer, SizeOf(Buffer));
  Result := CopyBufToStr(Buffer);
  end;

function TNdfdLib.LoadForecast(const Latitude  : string;
                               const Longitude : string;
                                     GetMetric : Boolean) : Boolean;
begin
  Result := NdfdLoadForecast(NdfdObj, NdfdPChar(AnsiString(Latitude)), 
                             NdfdPChar(AnsiString(Longitude)), GetMetric);
end;

function TNdfdLib.GetMaxTemp(DayNum : LongWord) : string;
begin
  if NdfdGetMaxTemp(NdfdObj, DayNum, Buffer, SizeOf(Buffer)) then
    Result := CopyBufToStr(Buffer)
  else
    Result := '';
end;

function TNdfdLib.GetMinTemp(DayNum : LongWord) : string;
begin
  if NdfdGetMinTemp(NdfdObj, DayNum, Buffer, SizeOf(Buffer)) then
    Result := CopyBufToStr(Buffer)
  else
    Result := '';
end;


end.
