unit NdfdSoap;

{

  Function that retrieves NDFD weather forecast data via SOAP service.

}

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  SysUtils,
  DateUtils,
{$IFDEF FPC}
 {$IF DEFINED(UseNSHttpClient)}
  ns_http_protocol,  //use iOS/OS X Foundation-based HTTP client
 {$ELSEIF DEFINED(UseSynapseHttpClient)}
  synapse_http_protocol,
  ssl_openssl,
 {$ELSEIF DEFINED(UseIndyHttpClient)}
  indy_http_protocol,
 {$ELSE}
  fpc_http_protocol,  //use FPC's HTTP client by default if nothing defined
 {$IFEND}
{$ELSE}  {Delphi - use Indy}
  delphi_init_com,
  indy_http_protocol,
{$ENDIF}
  soap_formatter,
  base_service_intf,
  ndfdXML,  //Web service units generated by WST's ws_helper
  ndfdXML_proxy;

function GetNdfdForecast(const Latitude  : string;
                         const Longitude : string;
                         const StartDate : string;
                         const NumDays   : Integer;
                         const GetMetric : Boolean;
                           out FcstXml   : string;
                           out ErrorMsg  : string) : Boolean;


implementation

function GetNdfdForecast(const Latitude  : string;
                         const Longitude : string;
                         const StartDate : string;
                         const NumDays   : Integer;
                         const GetMetric : Boolean;
                           out FcstXml   : string;
                           out ErrorMsg  : string) : Boolean;
 {Latitude and longitude are in decimal degrees.
  StartDate is in format yyyy-mm-dd or pass a blank string
   to start with today.
  NumDays is the number of forecast days to request.
  If GetMetric is True, return temperatures in Centigrade;
   if GetMetric is False, return temperatures in Fahrenheit.
  The XML returned by the service is returned in FcstXml.
  If successful, return True as function result; if error, 
   return error in ErrorMsg and False as function result.}

var
  NdfdService : ndfdXMLPortType;
  DateToUse   : TDateRemotable;
  UnitsToUse  : string;
  DataFmt     : TFormatSettings;

begin
  Result := False;

{$IFDEF FPC}
 {$IF DEFINED(UseNSHttpClient)}
  NS_RegisterHTTP_Transport;
 {$ELSEIF DEFINED(UseSynapseHttpClient)}
  Synapse_RegisterHTTP_Transport;
 {$ELSEIF DEFINED(UseIndyHttpClient)}
  INDY_RegisterHTTP_Transport;
 {$ELSE}
  FPC_RegisterHTTP_Transport;
 {$IFEND}
{$ELSE}
  INDY_RegisterHTTP_Transport;
{$ENDIF}

  try
    NdfdService := wst_CreateInstance_ndfdXMLPortType;
    if not Assigned(NdfdService) then
      begin
      ErrorMsg := 'Unable to connect to NDFD Web service.';
      Exit;
      end;

    DataFmt.DecimalSeparator := '.';
    DataFmt.DateSeparator := '-';
    DataFmt.ShortDateFormat := 'yyyy-mm-dd';

    DateToUse := TDateRemotable.Create;
    if StartDate <> '' then
      DateToUse.AsDate := StrToDate(StartDate, DataFmt)
    else
      DateToUse.AsDate := Today;
    if GetMetric then
      UnitsToUse := 'm'
    else 
      UnitsToUse := 'e';
    try
      FcstXml := NdfdService.NDFDgenByDay(StrToFloatDef(Latitude, 0, DataFmt),
                                          StrToFloatDef(Longitude, 0, DataFmt), 
                                          DateToUse, NumDays, UnitsToUse, 
                                          '24 hourly');
//      WriteLn(FcstXml);
    finally
      DateToUse.Free;
    end;

    Result := True;

  except on E:Exception do
    begin
    ErrorMsg := E.Message;
    Exit;
    end;
  end;

end;


end.
