library ndfd;

{
  Library that wraps TNdfdForecast class.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

uses
{$IFDEF UNIX}
  cthreads,
{$ENDIF}
  SysUtils,
  ndfd_types,
  NdfdForecast;

 //By default, use cdecl on Windows too: if stdcall, MSVC won't link
 // to 32-bit Delphi/FPC DLL unless export decorated names too.
{$DEFINE USE_CDECL}
{$IFDEF USE_STDCALL}  //Override and use stdcall?
  {$UNDEF USE_CDECL}
{$ENDIF}

{$I ndfdvers.inc}

procedure NdfdGetLibVersion(VerBuf    : NdfdPChar;
                            VerBufLen : NdfdUInt); {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Return library's version in buffer.}
begin
  CopyStrToBuf(NdfdLibVersion, VerBuf, VerBufLen);
end;


function NdfdInit : NdfdPObj; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Create and return a forecast object.
  Pass the returned object into other functions.}
begin
  Result := TNdfdForecast.Create;
end;


function NdfdUninit(NdfdObj : NdfdPObj) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Free forecast object.}
begin
  Result := False;
  if Assigned(NdfdObj) then
    begin
    TNdfdForecast(NdfdObj).Free;
    Result := True;
    end;
end;


procedure NdfdGetLastError(NdfdObj   : NdfdPObj;
                           ErrBuf    : NdfdPChar;
                           ErrBufLen : NdfdUInt); {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Return last error message in buffer.}
begin
  if Assigned(NdfdObj) then
    CopyStrToBuf(TNdfdForecast(NdfdObj).ErrorMsg, ErrBuf, ErrBufLen)
  else
    CopyStrToBuf('Object is nil.', ErrBuf, ErrBufLen);
end;


function NdfdLoadForecast(NdfdObj   : NdfdPObj;
                          Latitude  : NdfdPChar;
                          Longitude : NdfdPChar;
                          GetMetric : NdfdBool) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Load forecast for point with specified latitude and longitude.
  If GetMetric is False, return data in U.S. units (eg, Fahrenheit);
   if GetMetric is True, return data in metric units (eg, Centigrade).
  Returns True if load is successful; if load fails, returns False and
   error message can be retrieved with NdfdGetLastError.}
begin
  Result := False;
  if not Assigned(NdfdObj) then
    Exit;
  Result := TNdfdForecast(NdfdObj).LoadForecast(CopyBufToStr(Latitude),
                                                CopyBufToStr(Longitude),
                                                GetMetric);
end;


function NdfdGetMaxTemp(NdfdObj    : NdfdPObj;
                        DayNum     : NdfdUInt;
                        TempBuf    : NdfdPChar;
                        TempBufLen : NdfdUInt) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Copy forecast's high temperature for specified day (1=today) into buffer.
  Returns True if successful.}
begin
  Result := False;
  if not Assigned(NdfdObj) then
    Exit;
  CopyStrToBuf(TNdfdForecast(NdfdObj).MaxTemp[DayNum], TempBuf, TempBufLen);
  Result := True;
end;


function NdfdGetMinTemp(NdfdObj    : NdfdPObj;
                        DayNum     : NdfdUInt;
                        TempBuf    : NdfdPChar;
                        TempBufLen : NdfdUInt) : NdfdBool; {$IFDEF USE_CDECL}cdecl{$ELSE}stdcall{$ENDIF};
 {Copy forecast's low temperature for specified day (1=today) into buffer.
  Returns True if successful.}
begin
  Result := False;
  if not Assigned(NdfdObj) then
    Exit;
  CopyStrToBuf(TNdfdForecast(NdfdObj).MinTemp[DayNum], TempBuf, TempBufLen);
  Result := True;
end;


exports
  NdfdGetLibVersion,
  NdfdInit,
  NdfdUninit,
  NdfdGetLastError,
  NdfdLoadForecast,
  NdfdGetMaxTemp,
  NdfdGetMinTemp;

end.
