unit ndfd_types;

{
  Types passed to and from the ndfd library.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  SysUtils
{$IFDEF FPC},
  ctypes
{$ENDIF};


{$IFNDEF FPC}
type
  cint32  = LongInt;
  cuint32 = LongWord;
  cbool   = LongBool;
  {$IFDEF CPUX64}  //don't use NativeUInt!
  PtrUInt = UInt64;
  {$ELSE}
  PtrUInt = LongWord;
  {$ENDIF}
{$ENDIF}

type  {Library parameter types}
  NdfdChar  = AnsiChar;   {Single-byte character}
  NdfdPChar = PAnsiChar;  {32/64-bit pointer, depending on target, to
                            null-terminated string of single-byte characters}
  NdfdInt   = cint32;     {32 bits, signed}
  NdfdUInt  = cuint32;    {32 bits, unsigned}
  NdfdPObj  = Pointer;    {32/64-bit pointer, depending on target}
  NdfdBool  = cbool;      {32 bits, where False = 0 and True = -1}

procedure CopyStrToBuf(const AStr      : string;
                             StrBuf    : NdfdPChar;
                             StrBufLen : NdfdUInt);

function CopyBufToStr(StrBuf : NdfdPChar) : string;


implementation

procedure CopyStrToBuf(const AStr      : string;
                             StrBuf    : NdfdPChar;
                             StrBufLen : NdfdUInt);
 {Copy string's characters into buffer.
  If need to return non-ASCII characters to calling code,
   can do any conversion here.}
begin
  if Assigned(StrBuf) and (StrBufLen > 0) then
    StrPCopy(StrBuf, Copy(AnsiString(AStr), 1, StrBufLen-1));
end;


function CopyBufToStr(StrBuf : NdfdPChar) : string;
 {Return characters in buffer as a string.
  If calling code will be passing in any non-ASCII characters,
   can do any conversion here.}
begin
  Result := AnsiString(StrBuf);
end;


end.
